@@ -35,7 +35,6 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
-#include <X11/XKBlib.h>
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 
@@ -196,12 +195,9 @@ do_user_switch (GSLockPlug *plug)
 	gboolean res;
 	char    *command;
 
-	gboolean found;
-	found = is_program_in_path (MDM_FLEXISERVER_COMMAND);
-
-	if (found)
+	if (is_program_in_path (MDM_FLEXISERVER_COMMAND))
 	{
-
+		/* MDM */
 		command = g_strdup_printf ("%s %s",
 								   MDM_FLEXISERVER_COMMAND,
 								   MDM_FLEXISERVER_ARGS);
@@ -219,34 +215,56 @@ do_user_switch (GSLockPlug *plug)
 			g_error_free (error);
 		}
 	}
-	else {
-		
-		/* MDM not found, so we try to use gdmflexiserver from GDM */
-		
-		found = is_program_in_path (GDM_FLEXISERVER_COMMAND);
-		
-		if (found)
-		{
-                
-			char    *gdm_command;
+	else if (is_program_in_path (GDM_FLEXISERVER_COMMAND))
+	{
+		/* GDM */
+		command = g_strdup_printf ("%s %s",
+								   GDM_FLEXISERVER_COMMAND,
+								   GDM_FLEXISERVER_ARGS);
 
-			gdm_command = g_strdup_printf ("%s %s",
-										   GDM_FLEXISERVER_COMMAND,
-										   GDM_FLEXISERVER_ARGS);
-			
-			error = NULL;
-			res = gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
-											gdm_command,
-											&error);
+		error = NULL;
+		res = gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
+												command,
+												&error);
 
-			g_free (gdm_command);
-			
-			if (! res) {
-				gs_debug ("Unable to start MDM greeter: %s", error->message);
-				g_error_free (error);
-			}
+		g_free (command);
+		
+		if (! res) {
+			gs_debug ("Unable to start GDM greeter: %s", error->message);
+			g_error_free (error);
 		}
 	}
+	else if (g_getenv ("XDG_SEAT_PATH") != NULL)
+	{
+		/* LightDM */
+		GDBusProxyFlags flags = G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START;
+		GDBusProxy *proxy = NULL;
+
+		error = NULL;
+		proxy = g_dbus_proxy_new_for_bus_sync(G_BUS_TYPE_SYSTEM,
+											  flags,
+											  NULL,
+											  "org.freedesktop.DisplayManager",
+											  g_getenv ("XDG_SEAT_PATH"),
+											  "org.freedesktop.DisplayManager.Seat",
+											  NULL,
+											  &error);
+		if (proxy != NULL) {
+			g_dbus_proxy_call_sync (proxy,
+									"SwitchToGreeter",
+									g_variant_new ("()"),
+									G_DBUS_CALL_FLAGS_NONE,
+									-1,
+									NULL,
+									NULL);
+			g_object_unref (proxy);
+		}
+		else {
+			gs_debug ("Unable to start LightDM greeter: %s", error->message);
+			g_error_free (error);
+		}
+	}
+
 }
 
 static void
@@ -377,16 +395,17 @@ capslock_update (GSLockPlug *plug,
 static gboolean
 is_capslock_on (void)
 {
-	XkbStateRec states;
-	Display    *dsp;
+	GdkKeymap *keymap;
+	gboolean res;
 
-	dsp = GDK_DISPLAY ();
-	if (XkbGetState (dsp, XkbUseCoreKbd, &states) != Success)
-	{
-		return FALSE;
+	res = FALSE;
+
+	keymap = gdk_keymap_get_default ();
+	if (keymap != NULL) {
+		res = gdk_keymap_get_caps_lock_state (keymap);
 	}
 
-	return (states.locked_mods & LockMask) != 0;
+	return res;
 }
 
 static void
@@ -481,6 +500,13 @@ run_destroy_handler (GSLockPlug *plug,
 	ri->destroyed = TRUE;
 }
 
+static void
+run_keymap_handler (GdkKeymap *keymap,
+                    GSLockPlug *plug)
+{
+	capslock_update (plug, is_capslock_on ());
+}
+
 /* adapted from GTK+ gtkdialog.c */
 int
 gs_lock_plug_run (GSLockPlug *plug)
@@ -491,6 +517,8 @@ gs_lock_plug_run (GSLockPlug *plug)
 	gulong unmap_handler;
 	gulong destroy_handler;
 	gulong delete_handler;
+	gulong keymap_handler;
+	GdkKeymap *keymap;
 
 	g_return_val_if_fail (GS_IS_LOCK_PLUG (plug), -1);
 
@@ -507,6 +535,14 @@ gs_lock_plug_run (GSLockPlug *plug)
 		gtk_widget_show (GTK_WIDGET (plug));
 	}
 
+	keymap = gdk_keymap_get_for_display (gtk_widget_get_display (GTK_WIDGET (plug)));
+
+	keymap_handler =
+	    g_signal_connect (keymap,
+	                      "state-changed",
+	                      G_CALLBACK (run_keymap_handler),
+	                      plug);
+
 	response_handler =
 	    g_signal_connect (plug,
 	                      "response",
@@ -552,6 +588,7 @@ gs_lock_plug_run (GSLockPlug *plug)
 		g_signal_handler_disconnect (plug, unmap_handler);
 		g_signal_handler_disconnect (plug, delete_handler);
 		g_signal_handler_disconnect (plug, destroy_handler);
+		g_signal_handler_disconnect (plug, keymap_handler);
 	}
 
 	g_object_unref (plug);
@@ -1009,26 +1046,25 @@ gs_lock_plug_set_switch_enabled (GSLockPlug *plug,
 
 	if (switch_enabled)
 	{
-		gboolean found;
-		found = is_program_in_path (MDM_FLEXISERVER_COMMAND);
-		if (found)
+		if (is_program_in_path (MDM_FLEXISERVER_COMMAND))
 		{
+			/* MDM  */
+			gtk_widget_show (plug->priv->auth_switch_button);
+		}
+		else if (is_program_in_path (GDM_FLEXISERVER_COMMAND))
+		{
+			/* GDM */
+			gtk_widget_show (plug->priv->auth_switch_button);
+		}
+		else if (g_getenv ("XDG_SEAT_PATH") != NULL)
+		{
+			/* LightDM */
 			gtk_widget_show (plug->priv->auth_switch_button);
 		}
 		else
 		{
-			gs_debug ("Warning: MDM flexiserver command not found: %s", MDM_FLEXISERVER_COMMAND);
-			
-			/* MDM not found, so we try to use gdmflexiserver from GDM */
-			found = is_program_in_path (GDM_FLEXISERVER_COMMAND);
-			if (found)
-			{
-				gtk_widget_show (plug->priv->auth_switch_button);
-			}
-			else {
-				gs_debug ("Warning: GDM flexiserver command not found: %s", GDM_FLEXISERVER_COMMAND);
-				gtk_widget_hide (plug->priv->auth_switch_button);
-			}
+			gs_debug ("Warning: Unknown DM for switch button");
+			gtk_widget_hide (plug->priv->auth_switch_button);
 		}
 	}
 	else
@@ -1372,17 +1408,8 @@ entry_key_press (GtkWidget   *widget,
                  GdkEventKey *event,
                  GSLockPlug  *plug)
 {
-	gboolean capslock_on;
-
 	restart_cancel_timeout (plug);
 
-	capslock_on = is_capslock_on ();
-
-	if (capslock_on != plug->priv->caps_lock_on)
-	{
-		capslock_update (plug, capslock_on);
-	}
-
 	/* if the input widget is visible and ready for input
 	 * then just carry on as usual
 	 */
