@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Authors: William Jon McCann <mccann@jhu.edu>
  *
@@ -26,8 +26,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 
-#include <mateconf/mateconf-engine.h>
-#include <mateconf/mateconf-client.h>
+#include <gio/gio.h>
 
 #define MATE_DESKTOP_USE_UNSTABLE_API
 #include <libmateui/mate-bg.h>
@@ -54,9 +53,7 @@ struct GSManagerPrivate
 	GHashTable  *jobs;
 
 	GSThemeManager *theme_manager;
-	MateConfClient    *client;
 	MateBG        *bg;
-	guint           bg_notify_id;
 
 	/* Policy */
 	glong        lock_timeout;
@@ -1034,66 +1031,6 @@ on_bg_changed (MateBG   *bg,
 	gs_debug ("background changed");
 }
 
-static void
-mateconf_changed_callback (MateConfClient *client,
-                           guint        cnxn_id,
-                           MateConfEntry  *entry,
-                           GSManager   *manager)
-{
-	mate_bg_load_from_preferences (manager->priv->bg,
-	                               manager->priv->client);
-}
-
-static void
-watch_bg_preferences (GSManager *manager)
-{
-	g_assert (manager->priv->bg_notify_id == 0);
-
-	mateconf_client_add_dir (manager->priv->client,
-	                         MATE_BG_KEY_DIR,
-	                         MATECONF_CLIENT_PRELOAD_NONE,
-	                         NULL);
-	manager->priv->bg_notify_id = mateconf_client_notify_add (manager->priv->client,
-	                              MATE_BG_KEY_DIR,
-	                              (MateConfClientNotifyFunc)mateconf_changed_callback,
-	                              manager,
-	                              NULL,
-	                              NULL);
-}
-
-static MateConfClient *
-get_mateconf_client (void)
-{
-	MateConfClient        *client;
-	GSList             *addresses;
-	GError             *error;
-	MateConfEngine        *engine;
-
-	client = NULL;
-	addresses = NULL;
-
-	addresses = g_slist_prepend (addresses, "xml:merged:" SYSCONFDIR "/mateconf/mateconf.xml.mandatory");
-	addresses = g_slist_prepend (addresses, "xml:merged:" SYSCONFDIR "/mateconf/mateconf.xml.system");
-	addresses = g_slist_prepend (addresses, "xml:merged:" SYSCONFDIR "/mateconf/mateconf.xml.defaults");
-	addresses = g_slist_reverse (addresses);
-
-	error = NULL;
-	engine = mateconf_engine_get_for_addresses (addresses, &error);
-	if (engine == NULL)
-	{
-		gs_debug ("Unable to get mateconf engine for addresses: %s", error->message);
-		g_error_free (error);
-	}
-	else
-	{
-		client = mateconf_client_get_for_engine (engine);
-	}
-
-	g_slist_free (addresses);
-
-	return client;
-}
-
 static void
 gs_manager_init (GSManager *manager)
 {
@@ -1103,19 +1040,14 @@ gs_manager_init (GSManager *manager)
 	manager->priv->grab = gs_grab_new ();
 	manager->priv->theme_manager = gs_theme_manager_new ();
 
-	manager->priv->client = get_mateconf_client ();
-	if (manager->priv->client != NULL)
-	{
-		manager->priv->bg = mate_bg_new ();
+	manager->priv->bg = mate_bg_new ();
 
-		g_signal_connect (manager->priv->bg,
-		                  "changed",
-		                  G_CALLBACK (on_bg_changed),
-		                  manager);
-		watch_bg_preferences (manager);
+	g_signal_connect (manager->priv->bg,
+					  "changed",
+					  G_CALLBACK (on_bg_changed),
+					  manager);
 
-		mate_bg_load_from_preferences (manager->priv->bg, manager->priv->client);
-	}
+	mate_bg_load_from_system_preferences (manager->priv->bg);
 }
 
 static void
@@ -1726,23 +1658,10 @@ gs_manager_finalize (GObject *object)
 
 	g_return_if_fail (manager->priv != NULL);
 
-	if (manager->priv->bg_notify_id != 0)
-	{
-		mateconf_client_remove_dir (manager->priv->client,
-		                            MATE_BG_KEY_DIR,
-		                            NULL);
-		mateconf_client_notify_remove (manager->priv->client,
-		                               manager->priv->bg_notify_id);
-		manager->priv->bg_notify_id = 0;
-	}
 	if (manager->priv->bg != NULL)
 	{
 		g_object_unref (manager->priv->bg);
 	}
-	if (manager->priv->client != NULL)
-	{
-		g_object_unref (manager->priv->client);
-	}
 
 	free_themes (manager);
 	g_free (manager->priv->logout_command);
